@@ -1,4 +1,4 @@
-nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
+nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE,SES=FALSE,RC=FALSE)
 {
   if(is.null(nst.result$details)){stop("PANOVA need detailed output of NST.")}
   rand.mean=nst.result$details$rand.mean
@@ -18,6 +18,20 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
   rm(nst.result)
   obs=as.numeric(obs3[,3])
   
+  if(SES)
+  {
+    rand.sd=apply(dist.ran,1,stats::sd,na.rm=TRUE)
+    sesb=(obs-rand.mean)/rand.sd
+    sesb[which(obs==rand.mean)]=0
+  }else{sesb=NULL}
+  if(RC)
+  {
+    obsmm=matrix(obs,nrow=nrow(dist.ran),ncol=ncol(dist.ran))
+    a1=rowSums(dist.ran>obsmm,na.rm = TRUE)/ncol(dist.ran)
+    a2=rowSums(dist.ran==obsmm,na.rm = TRUE)/ncol(dist.ran)
+    rcb=(0.5-(a1+(a2/2)))*2
+  }else{rcb=NULL}
+  
   GD=rand.mean/cbind(dist.ran,obs) # G / D and G / Gk
   EC=(Dmax-rand.mean)/(Dmax-cbind(dist.ran,obs)) # E / C and E / Ek
   EC[is.nan(EC)]=1
@@ -28,8 +42,20 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
   Eij=(Dmax-rand.mean)/Dmax
   Dsij = obs/Dmax
   Gsij = rand.mean/Dmax
-  MSTsij = (Eij/Cij) * (Dsij/Gsij)
-  MSTsij[which(MSTsij > 1)] = ((Cij/Eij) * (Gsij/Dsij))[which(MSTsij > 1)]
+  
+  ####
+  ECijx=Eij/Cij; CEijx=Cij/Eij
+  ECijx[is.nan(ECijx)]=1
+  CEijx[is.nan(CEijx)]=1
+  DGijx=Dsij/Gsij;GDijx=Gsij/Dsij
+  DGijx[is.nan(DGijx)]=1
+  GDijx[is.nan(GDijx)]=1
+  
+  MSTsij = (ECijx) * (DGijx)
+  MSTsij[which(MSTsij > 1)] = ((CEijx) * (GDijx))[which(MSTsij > 1)]
+  #####
+  #MSTsij = (Eij/Cij) * (Dsij/Gsij)
+  #MSTsij[which(MSTsij > 1)] = ((Cij/Eij) * (Gsij/Dsij))[which(MSTsij > 1)]
   
   STmin=Eij
   STmin[which(obs>rand.mean)]=(1-Eij)[which(obs>rand.mean)]
@@ -62,16 +88,22 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
   eps=.Machine$double.eps
   if(trace){trace.seq=seq(from=1,to=rands[[1]],by=200)}else{trace.seq=NULL}
   
-  stnst<-function(u,perms,rands,groups,obs3,ECGD,STmin,trace.seq,MSTsij)
+  stnst<-function(u,perms,rands,groups,obs3,ECGD,STmin,trace.seq,MSTsij,SES,RC,sesb,rcb)
   {
     if(!is.null(trace.seq)){if(u %in% trace.seq){message("Now randomization u=",u," in ",rands[[1]],". ",date())}}
     STs<-NSTs<-MSTs<-grps<-STij<-NSTij<-MSTij<-grpij<-list()
+    if(SES){SESs<-SESij<-list()}
+    if(RC){RCs<-RCij<-list()}
     for(k in 1:length(perms))
     {
       STs[[k]]<-NSTs[[k]]<-MSTs[[k]]<-grps[[k]]<-STij[[k]]<-NSTij[[k]]<-MSTij[[k]]<-grpij[[k]]<-list()
+      if(SES){SESs[[k]]<-SESij[[k]]<-list()}
+      if(RC){RCs[[k]]<-RCij[[k]]<-list()}
       if(u>nrow(perms[[k]]))
       {
         STs[[k]]<-NSTs[[k]]<-MSTs[[k]]<-grps[[k]]<-STij[[k]]<-NSTij[[k]]<-MSTij[[k]]<-grpij[[k]]<-NA
+        if(SES){SESs[[k]]<-SESij[[k]]<-NA}
+        if(RC){RCs[[k]]<-RCij[[k]]<-NA}
       }else{
         groupk=groups[[k]]
         rownames(groupk)=perms[[k]][u,]
@@ -97,6 +129,9 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
           NSTs[[k]][[i]]=NSTi
           MSTs[[k]][[i]]=mean(MSTsij[idi],na.rm = TRUE)
           grps[[k]][[i]]=grpk.lev[i]
+          
+          if(SES){SESij[[k]][[i]]=sesb[idi];SESs[[k]][[i]]=mean(sesb[idi],na.rm = TRUE)}
+          if(RC){RCij[[k]][[i]]=rcb[idi];RCs[[k]][[i]]=mean(rcb[idi],na.rm = TRUE)}
         }
       }
     }
@@ -110,10 +145,16 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
           Fobs[[k]]<-Pobs[[k]]<-dobs[[k]]<-NA
           Rname[[k]]<-Vobs[[k]]<-c(NA,NA)
         }else{
-          aovs=stats::aov(unlist(Vij[[k]])~unlist(grpij[[k]]))
-          aovss=summary(aovs)
-          Fobs[[k]]=aovss[[1]]$`F value`[1]
-          Pobs[[k]]=aovss[[1]]$`Pr(>F)`[1]
+          aovs=try(stats::aov(unlist(Vij[[k]])~unlist(grpij[[k]])),silent = TRUE)
+          if(class(aovs)[1]=="try-error")
+          {
+            Fobs[[k]]=NA
+            Pobs[[k]]=NA
+          }else{
+            aovss=summary(aovs)
+            Fobs[[k]]=aovss[[1]]$`F value`[1]
+            Pobs[[k]]=aovss[[1]]$`Pr(>F)`[1]
+          }
           if(k==1){dobs[[k]]=NA}else{dobs[[k]]=Vi[[k]][[1]]-Vi[[k]][[2]]}
           if(k==1){Rname[[k]]=c("all.grp","all.grp")}else{Rname[[k]]=c(grpi[[k]][[1]],grpi[[k]][[2]])}
           if(k==1){Vobs[[k]]=c(NA,NA)}else{Vobs[[k]]=c(Vi[[k]][[1]],Vi[[k]][[2]])}
@@ -126,13 +167,19 @@ nst.panova<-function(nst.result,group=NULL,rand=999,trace=TRUE)
       rownames(Vout)=c();colnames(Vout)=c("group1","group2",paste0("Index.",c("group1","group2")))
       list(Vobs=Vout,Fvalue=Fobs,Pvalue=Pobs,dif=dobs)
     }
-    list(ST.dfp=dFP(Vij=STij,grpij=grpij,Vi=STs,grpi=grps),
+    outx=list(ST.dfp=dFP(Vij=STij,grpij=grpij,Vi=STs,grpi=grps),
          NST.dfp=dFP(Vij=NSTij,grpij=grpij,Vi=NSTs,grpi=grps),
          MST.dfp=dFP(Vij=MSTij,grpij=grpij,Vi=MSTs,grpi=grps))
+    if(SES){outx=c(outx,list(SES.dfp=dFP(Vij=SESij,grpij=grpij,Vi=SESs,grpi=grps)))}
+    if(RC){outx=c(outx,list(RC.dfp=dFP(Vij=RCij,grpij=grpij,Vi=RCs,grpi=grps)))}
+    outx
   }
   
-  rres=lapply(1:rands[[1]],stnst,perms,rands,groups,obs3,ECGD,STmin,trace.seq,MSTsij)
+  rres=lapply(1:rands[[1]],stnst,perms,rands,groups,obs3,ECGD,STmin,trace.seq,MSTsij,SES,RC,sesb,rcb)
   ind.names=c("ST","NST","MST")
+  if(SES){ind.names=c(ind.names,"SES")}
+  if(RC){ind.names=c(ind.names,"RC")}
+  
   output=list()
   for(j in 1:length(rres[[1]]))
   {
